from multiprocessing.dummy import Pool


class Semestre:
    def __init__(self, date, discs):
        self.date = date
        self.discs = discs

    def update(self, no_jobs=None):
        pool = Pool(len(self.discs)) if no_jobs == None else Pool(no_jobs)
        for _ in pool.imap(lambda disc: disc.update(), self.discs):
            pass



class Nota:
    def __init__(self, nome, individual, turma):
        self.nome = nome if nome else None
        self.individual = float(individual) if individual else None
        self.turma = float(turma) if turma else None

    def __repr__(self):
        return f"Nota({repr(self.nome)},{self.individual},{self.turma})"

    def _parser(row):
        t = tuple(elem.text.strip().replace(",", ".") for elem in row.select("span"))
        return Nota(*t) if len(t) == 3 else None



from bs4 import BeautifulSoup as BS
from . import Nota, Semestre, Frequencia
import requests
import html2markdown
import re
from itertools import chain
from collections import Counter
from .. import HEADERS
from ..utils import parse_table


class Disciplina:
    __request_getter = None

    def __init__(
        self,
        base_url,
        cakephp_cookie,
        nome,
        ch,
        _id,
        notas=None,
        turma=None,
        prof=None,
        situacao=None,
        formula=None,
        ementa=None,
        frequencias=None,
    ):
        self.__id = _id
        self.__nome = nome
        self.__ch = ch
        self.__is_cached = False
        self.__sess = requests.Session()
        self.__sess.headers = HEADERS
        cookie_obj = requests.cookies.create_cookie(
            domain="siscad.ufms.br", name="CAKEPHP", value=cakephp_cookie
        )
        self.__sess.cookies.set_cookie(cookie_obj)
        self.__base_url = base_url
        # Requerem self.update()
        # --------------------------------------------------------------
        self.__notas = notas
        self.__turma = turma
        self.__prof = prof
        self.__situacao = situacao
        self.__formula = formula
        self.__ementa = ementa
        self.__frequencias = frequencias

    def update(self):
        self.__is_cached |= True
        res = self.__sess.get(f"{self.base_url}/academico/disciplinas/{self.id}")
        self.__parser(res.text)

    def get_faltas(self):
        freqs = Counter(
            chain.from_iterable([f.chamadas for f in self.frequencias])
        ).get(False, 0)
        return freqs * 100 / self.ch

    def get_presencas(self):
        return 100 - self.get_faltas()

    def __repr__(self):
        return f"""
                Disciplina(
                    base_url={repr(self.__base_url)},
                    cakephp_cookie={repr(self.__sess.cookies.get("CAKEPHP"))},
                    nome={repr(self.__nome)},
                    ch={repr(self.__ch)},
                    id={repr(self.__id)},
                    notas={repr(self.__notas)},
                    turma={repr(self.__turma)},
                    prof={repr(self.__prof)},
                    situacao={repr(self.__situacao)},
                    formula={repr(self.__formula)},
                    ementa={repr(self.__ementa)},
                    frequencias={repr(self.__frequencias)}

                )
            """.replace(
            "\n", ""
        ).replace(
            "\t", ""
        )

    def __parser(self, doc):
        soup = BS(doc, "html.parser")

        table = soup.select_one("div.box.box-primary.collapsed-box")
        table = table.select("table")[1]
        entry = parse_table(table)[0]
        self.__turma = entry["Turma"].text.strip()
        self.__prof = entry["Docentes"].text.strip()

        avaliacoes = soup.select_one("div#avaliacoes")

        situacao = avaliacoes.select_one("div.row.color-gray").select_one("span")
        situacao.select_one("strong").extract()
        self.__situacao = situacao.text.strip()

        ma = avaliacoes.select_one("div#ma")

        formula = ma.select("span")
        self.__formula = formula[1].text.strip() if len(formula) >= 2 else None

        mf = Nota._parser(ma.select_one("div.row"))

        self.__notas = [mf] if mf else []
        divs = (
            avaliacoes.select("div.box.box-primary")[1]
            .select_one("div.row")
            .find_all("div", recursive=False)
        )
        for div in divs:
            for row in div.select("div.row")[1:-1]:
                nota = Nota._parser(row)
                if nota:
                    self.__notas.append(nota)

        frequencias = soup.select_one("div#frequencias")

        self.__frequencias = []

        for div in frequencias.select("div.box.box-primary")[1:]:
            year = div.select_one("div").text.strip().split("/")[-1]
            table = div.select_one("table>tbody")
            for tr in table.select("tr"):
                day_month, hour_minute, tipo, *chamadas = (
                    elem.text.strip() for elem in tr.select("td")
                )
                self.__frequencias.append(
                    Frequencia._parser(year, day_month, hour_minute, tipo, chamadas)
                )

        planos = soup.select_one("div#planos")
        for h3 in planos.select("h3"):
            del h3.attrs["class"]
        planos = "".join(re.compile(r"<div.*>|<\/div>|\n+|\t+").split(str(planos)))
        self.__ementa = html2markdown.convert(planos)

    def __getattr__(self, attr):
        try:
            value = getattr(self, "_Disciplina__" + attr)
        except Exception:
            raise AttributeError(
                f"AttributeError: '{type(self)}' object has no attribute '{attr}'"
            )
        if not self.__is_cached:
            self.update()
        value = getattr(self, "_Disciplina__" + attr)
        if value == None:
            raise RuntimeError("Unexpected error")
        return value



from collections import namedtuple
from datetime import datetime


class Frequencia(namedtuple("Frequencia", ["dt", "tipo", "chamadas"])):
    def _parser(year, day_month, hour_minute, tipo, chamadas):
        day, month = day_month.split("/")
        hour, minute = hour_minute.split(":")
        dt = datetime(
            year=int(year),
            day=int(day),
            month=int(month),
            hour=int(hour),
            minute=int(minute),
        )
        chamadas = [
            True if c == "P" else (False if c == "F" else None) for c in chamadas
        ]
        return Frequencia(dt, tipo, chamadas)



def parse_table(elem):
    """
    Parses HTML and return a list of dicts with keys being the header entries
    """
    keys = [h.text.strip() for h in elem.find("thead").find_all("th")]
    ans = []
    for tr in elem.find("tbody").find_all("tr"):
        ans.append(dict(zip(keys, (elem for elem in tr.find_all("td")))))
    return ans



import requests
from bs4 import BeautifulSoup as BS
from urllib.parse import urljoin
import os
import re
from .model import Disciplina, Semestre
from . import HEADERS
from .utils import parse_table

regex_extract_number = lambda s: int(re.search(r"\d+", s).group(0))

base_url_join = lambda *args: urljoin(Siscad.base_url, os.path.join(*args))


class Siscad:
    base_url = "https://siscad.ufms.br"

    def __init__(self, passaporte, senha):
        """
        login no SISCAD
            passaporte:str
            senha:str
        atributos:
            rga:str
            nome:str
            carga_hor:float
            cr:float
            discs_feitas:int
            discs:list de tipo Disciplina
        """
        self.sess = requests.Session()
        self.sess.headers = HEADERS
        res = self.sess.post(
            base_url_join("login"), data={"passaporte": passaporte, "senha": senha}
        )
        if res.url != base_url_join("academico"):
            raise RuntimeError("Senha/Passaporte Invalido")
        self.__main_parser(res.text)

    def __request_getter(self, *args):
        return self.sess.get(base_url_join(*args))

    def get_semestres(self):
        res = self.__request_getter("academico", "disciplinas")
        if not res.ok:
            raise RuntimeError('Falha ao acessar endereço "%s"' % res.url)
        return self.__disciplinas_parser(res.text)

    def __disciplinas_parser(self, doc):
        soup = BS(doc, "html.parser")
        ans = []

        for div in soup.select("div.box-primary")[1:]:
            date = div.select_one("h3.box-title.color-gray").text.strip()
            sem = Semestre(re.search(r"\d+\/\d+", date).group(0), [])
            for d in parse_table(div.select_one("table")):
                try:
                    _id = regex_extract_number(
                        d["Código da Disciplina"].select_one("a").attrs["href"]
                    )
                    nome = d["Nome da Disciplina"].text.strip()
                    situacao = d["Situação"].text.strip()
                    ch = regex_extract_number(d["C.H."].text.strip())
                except KeyError as e:
                    print("keys available:", list(d.keys()))
                    raise e
                sem.discs.append(
                    Disciplina(
                        Siscad.base_url, self.sess.cookies.get("CAKEPHP"), nome, ch, _id
                    )
                )
            ans.append(sem)
        return ans[::-1]

    def __main_parser(self, doc):
        soup = BS(doc, "html.parser")
        # nome / rga
        self.rga = soup.select_one("li.user-header>p>small").text.strip().split(" ")[-1]
        self.nome = soup.select_one("div.pull-left.info>p").text
        # carga_hor,cr,discs_feitas
        box_content = soup.select("div.info-box-content")
        carga_hor, discs_feitas, mga, cr = (
            elem.select_one("span.info-box-number").text.strip() for elem in box_content
        )
        self.carga_hor = float(carga_hor.replace(",", ".")[:-1])
        self.cr = float(cr.replace(",", "."))
        self.mga = float(mga.replace(",", "."))
        self.discs_feitas = int(discs_feitas)
        # ultima_matricula
        self.ultima_matricula = (
            soup.select_one("h3.box-title.color-gray").text.split(":")[-1].strip()
        )



from pathlib import Path

script_path = Path(__file__).parent
__version__ = (script_path / "_version.py").read_text().strip()

# https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/User-Agent
# Mozilla/<version> (<system-information>) <platform> (<platform-details>) <extensions>
import requests
import platform

HEADERS = {
    "Accept-Encoding": "gzip, deflate",
    "Accept": "*/*",
    "Connection": "keep-alive",
    "User-Agent": f"python-requests/{requests.__version__} ({platform.platform()}) {__package__}-{__version__}",
    "pypi-url": "https://pypi.org/project/siscad",
}
from .Siscad import Siscad



from setuptools import setup, find_packages
from pathlib import Path

version = Path("./siscad/_version.py").read_text().strip()
setup(
    name="siscad",
    license="GPLv3",
    version=version,
    author="Giuliano Oliveira De Macedo",
    author_email="giuliano.llpinokio@gmail.com",
    description="Module for siscad.ufms.br",
    long_description=Path("README.md").read_text(),
    download_url=f"https://github.com/llpinokio/siscad.py/archive/v{version}.tar.gz",
    long_description_content_type="text/markdown",
    keywords=["web-scraping"],
    url="https://github.com/llpinokio/siscad.py",
    packages=find_packages(),
    install_requires=Path("requirements.txt").read_text().split(),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
)
